# ICASSP experient - Convergence

import os, sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

import warnings
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from srenet.data import sample_data
from srenet.projgrad import projgrad
from utils import get_params_from_string, get_label_from_string, get_color_from_string


################################ Configuration #################################

# This is the only part of the code you may have to edit

m, n = 100, 300         # Dictionnary dimension
dic_gen = 'gaussian'    # Dictionnary generation method : `gaussian`, `uniform`, `dct` or `toeplitz`
obs_gen = 'gaussian'    # Observation generation method : `gaussian` or `posgaussian`
l1ratio = 0.2           # Ratio lambda/lambda_max where lambda_max = max(A.T * y)
l2ratio = 0.5           # Ration epsilon/lambda_max where lambda_max = max(A.T * y)


################################## Experiment ##################################

print("-------------------------------")
print("ICASSP experiment - Convergence\n")

methods = ['aPG','aPGs','aPGr','S&R']
res = {method: {} for method in methods}

x0 = np.zeros(n)
A, y = sample_data(m, n, dic_gen, obs_gen)
lmax = np.max(A.T @ y)
l1 = l1ratio * lmax
l2 = l2ratio * lmax

for method in methods:
    print(f"Method {method} is running ...")
    params = get_params_from_string(method)
    params.maxit = 10_000
    params.tol = 1e-15
    params.verbosity = False
    x, mnt = projgrad(x0,A,y,l1,l2,params)
    res[method]['x'] = x
    res[method]['mnt'] = mnt


##############################   Plot results   ################################

plt.rcParams["font.size"] = 12
plt.rcParams["axes.labelsize"] = 14
plt.rcParams["axes.titlesize"] = 14
plt.rcParams["lines.linewidth"] = 2
plt.rcParams["lines.markersize"] = 5
plt.rcParams["xtick.labelsize"] = 12
plt.rcParams["ytick.labelsize"] = 12
plt.rcParams["text.usetex"] = True
plt.rcParams["font.family"] = "serif"
plt.rcParams["font.serif"] = ["Helvetica"]

fig, ax = plt.subplots(1, 1, figsize=(6,4))

for method in methods:
    ax.plot(
        res[method]['mnt'].flops,
        res[method]['mnt'].obj - res[method]['mnt'].obj[-1],
        label=get_label_from_string(method), 
        color=get_color_from_string(method),
        marker= '.',
    )
ax.set_xlabel(r"FLOPs")
ax.set_ylabel(r"Objective error")
ax.set_xscale('log')
ax.set_yscale('log')

plt.legend()
plt.grid(b=True, which='major', axis='y')
plt.grid(b=True, which='minor', axis='y', alpha=0.2)
plt.minorticks_on()
plt.tight_layout()
plt.show()

print("-------------------------------")