# ICASSP experient - Performance profiles

import os, sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

import warnings
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from srenet.data import sample_data
from srenet.projgrad import projgrad
from utils import get_params_from_string, get_label_from_string, get_color_from_string


################################ Configuration #################################

# This is the only part of the code you may have to edit

m, n = 100, 300         # Dictionnary dimension
dic_gen = 'gaussian'    # Dictionnary generation method : `gaussian`, `uniform`, `dct` or `toeplitz`
obs_gen = 'gaussian'    # Observation generation method : `gaussian` or `posgaussian`
l1ratio = 0.2           # Ratio lambda/lambda_max where lambda_max = max(A.T * y)
l2ratio = 0.5           # Ration epsilon/lambda_max where lambda_max = max(A.T * y)
flops = 2_000_000       # FLOPs budget
repeats = 100           # Number of instances tested


################################## Experiment ##################################

print("---------------------------------------")
print("ICASSP experiment - Performance profiles\n")

tolmax = 1.
tolmin = 1e-15
tolnb = 30
tols = np.logspace(np.log10(tolmax), np.log10(tolmin), tolnb)
methods = ['aPG','aPGs','aPGr','S&R']
solved = np.full((len(methods), tolnb, repeats), 0)

for rep in range(repeats):
    print(f"Repeat {rep+1}/{repeats}")

    x0   = np.zeros(n)
    A, y = sample_data(m, n, dic_gen, obs_gen)
    lmax = np.max(A.T @ y)
    l1   = l1ratio * lmax
    l2   = l2ratio * lmax

    for (mi,method) in enumerate(methods):
        try:
            print(f"  Method {method} is running ...")

            params = get_params_from_string(method)
            params.maxit = 10_000
            params.tol = tolmin
            params.verbosity = False
            
            x, mnt = projgrad(x0,A,y,l1,l2,params)

            if not mnt.stopcrit.has_converged:
                warnings.warn("Method did not converge.")
                continue

            for ti, tol in enumerate(tols):
                its_solved = np.where(mnt.gap < tol)[0]
                if len(its_solved) > 0:
                    it_solved = its_solved[0]
                    if mnt.flops[it_solved] < flops:
                        solved[mi,ti,rep] = 1
                    else:
                        solved[mi,ti,rep] = 0

        except Exception as e:
                warnings.warn("Repeat failed with the following error :")
                print(e)

solved = pd.DataFrame(
        np.nanmean(solved, axis=-1),
        columns = ['{:.2e}'.format(tol) for tol in tols],
        index = [method for method in methods],
    )

##############################   Plot results   ################################

plt.rcParams["font.size"] = 12
plt.rcParams["axes.labelsize"] = 18
plt.rcParams["axes.titlesize"] = 18
plt.rcParams["lines.linewidth"] = 2
plt.rcParams["lines.markersize"] = 5
plt.rcParams["xtick.labelsize"] = 12
plt.rcParams["ytick.labelsize"] = 12
plt.rcParams["text.usetex"] = True
plt.rcParams["font.family"] = "serif"
plt.rcParams["font.serif"] = ["Helvetica"]

fig, ax = plt.subplots(1, 1, figsize=(6,4))

for method in methods:
    ax.plot(
        tols, 
        solved.loc[method,:], 
        label=get_label_from_string(method), 
        color=get_color_from_string(method),
        marker= '.',
    )
ax.set_xlabel(r"$\tau$")
ax.set_ylabel(r"$\rho_{\%}$")
ax.set_xscale('log')
ax.yaxis.set_major_formatter(lambda x,pos: r"{0:.0f}\%".format(x * 100))

plt.legend()
plt.grid(b=True, which='major', axis='y')
plt.grid(b=True, which='minor', axis='y', alpha=0.2)
plt.minorticks_on()
plt.tight_layout()
plt.show()

print("---------------------------------------")